using Models.Model.Backend;

namespace Backend.Helper;

public static class FilesystemHelper
{
    public static DatabaseSizes GetDatabaseSizes(string basePath)
    {
        DatabaseSizes databaseSizes = new();
        
        FileInfo fileInfo = new($"{basePath}/Models/mydb.db");
        databaseSizes.MyDbSize = fileInfo.Length.ToSize(SizeUnits.KB);
        
        databaseSizes.DiscardedDbSize = GetDiscardedDbSizes(basePath).ToSize(SizeUnits.KB);
        
        fileInfo = new($"{basePath}/Models/Filtered.db");
        databaseSizes.FilteredDbSize = fileInfo.Length.ToSize(SizeUnits.KB);
        
        return databaseSizes;
    }

    private static long GetDiscardedDbSizes(string basePath)
    {
        string folder = $"{basePath}/Models";
        string[] files = Directory.GetFiles(folder, "*.db");
        
        long size = 0;
        
        for (int i = 0; i < files.Length; i++)
        {
            if (!files[i].Contains("Discarded")) continue;
            FileInfo fileInfo = new(files[i]);
            size += fileInfo.Length;
        }
        
        return size;
    }
    
    private static double ToSize(this long value, SizeUnits unit)
    {
        return double.Parse((value / Math.Pow(1024, (long)unit)).ToString("0.00"));
    }
}